/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <stdlib.h> /** system() */
#include <gtk/gtk.h>

#include "gui_top_area.h"
#include "gui_bottom_area.h"
#include "gui_middle_area.h"
#include "gui_status_icon.h"
#include "gui_popup_menu.h"
#include "gui_join_create_guide.h"
#include "callbacks.h"
#include "io_misc.h"
#include "io_ipc.h"
#include "main.h"

#define WINDOW_RESIZE_AREA 10
#define TOP_CURSOR_AREA_BUTTON_SIZE 20

void
top_area_left_button_clicked (GuiTopArea *top_area, MainPrivate* priv)
{
#if (GTK_MINOR_VERSION >= 10)
	gtk_window_get_position(GTK_WINDOW (priv->window), &priv->window_x, &priv->window_y);
	gtk_widget_hide_all(priv->window);
#else		
	gtk_window_iconify(GTK_WINDOW (priv->window));
#endif	
}

void
top_area_right_button_clicked (GuiTopArea *top_area, MainPrivate* priv)
{	
	gtk_main_quit ();
}

void
network_row_activated(GuiMiddleArea* middle_area, IPCNetwork *net, guint button, MainPrivate* priv)
{
	if(!priv->network_menu) priv->network_menu = gui_popup_menu_create_network_menu(priv);
	priv->network_menu_row = net;
	gtk_menu_popup(GTK_MENU(priv->network_menu), NULL, NULL, NULL, NULL, button, gtk_get_current_event_time());
}

void
client_row_activated(GuiMiddleArea* middle_area, IPCClient *client, guint button, MainPrivate* priv)
{
	if(!priv->client_menu) priv->client_menu = gui_popup_menu_create_client_menu(priv);
	priv->client_menu_row = client;
	gtk_menu_popup(GTK_MENU(priv->client_menu), NULL, NULL, NULL, NULL, button, gtk_get_current_event_time());
}

void
popup_menu_network_status_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{
	if(priv->network_menu_row->online)
		ipc_network_status(priv->connection, priv->network_menu_row->name, FALSE);	
	else
		ipc_network_status(priv->connection, priv->network_menu_row->name, TRUE);	
}

void
popup_menu_network_leave_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	ipc_network_leave(priv->connection, priv->network_menu_row->name);
}

void
popup_menu_network_delete_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	ipc_network_delete(priv->connection, priv->network_menu_row->name);
}

void
popup_menu_client_ping_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	gchar* cmd;
	cmd = g_strconcat("xterm", " -e ", "ping ", priv->client_menu_row->addr, NULL);
	io_misc_run_command(cmd);
	g_free(cmd);
}

void
popup_menu_client_browse_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	gchar* cmd;
	cmd = g_strconcat("nautilus", " smb://", priv->client_menu_row->addr, "/", NULL);
	io_misc_run_command(cmd);
	g_free(cmd);
}

void
popup_menu_client_vnc_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	gchar* cmd;
	cmd = g_strconcat("vncviewer", " ", priv->client_menu_row->addr, NULL);
	io_misc_run_command(cmd);
	g_free(cmd);
}

void
popup_menu_client_evict_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	ipc_network_evict(priv->connection, 
			 		  priv->client_menu_row->net->name, 
					  priv->client_menu_row->addr);
}

void
popup_menu_client_ssh_activate(GtkMenuItem *menuitem, MainPrivate* priv)
{	
	gchar* cmd;
	cmd = g_strconcat("xterm", " -e ", "ssh ", priv->client_menu_row->addr, NULL);
	io_misc_run_command(cmd);
	g_free(cmd);
}

void
bottom_area_stop_button_clicked (GuiBottomArea *bottom_area, int type, MainPrivate* priv)
{	
	switch(type){
	case GUI_BOTTOM_AREA_HAMACHI_STOP:
		#if (GTK_MINOR_VERSION >= 10)
		gui_status_icon_set_status(priv->status, FALSE);
		#endif

		priv->hamchi_user_stop = TRUE;
		ipc_shutdown(priv->connection);
		priv->connection = NULL;
		gui_middle_area_set_network(GUI_MIDDLE_AREA(priv->middle_area), NULL);
		gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), "Hamachi not running");
	break;
	case GUI_BOTTOM_AREA_HAMACHI_START:
		#if (GTK_MINOR_VERSION >= 10)
		gui_status_icon_set_status(priv->status, TRUE);
		#endif

		priv->hamchi_user_stop = FALSE;
	break;
	default:
	break;
	}
}

void
bottom_area_middle_button_clicked (GuiBottomArea *bottom_area, gint x, gint y, MainPrivate* priv)
{
	gui_join_create_guide_run(priv);
}

gboolean
main_window_button_press (GtkWidget *widget, GdkEventButton *event, MainPrivate* priv)
{
	if(event->button == 1){
		if(event->window == priv->bottom_area->window){
			if( (event->y > ((priv->bottom_area->allocation.height-1)-WINDOW_RESIZE_AREA)) &&
				(event->x > ((priv->bottom_area->allocation.width-1)-WINDOW_RESIZE_AREA))){
				gtk_window_begin_resize_drag(GTK_WINDOW (priv->window),
												GDK_WINDOW_EDGE_SOUTH_EAST,
												event->button, event->x_root,
												event->y_root, event->time);
				return TRUE;
			}
		}else if(event->window == priv->top_area->window){
			gtk_window_begin_move_drag (GTK_WINDOW (priv->window),
										event->button, event->x_root,
										event->y_root, event->time);
			return TRUE;
		}
	}

	return FALSE;
}

gboolean
main_window_motion_notify (GtkWidget *widget, GdkEventMotion *event, MainPrivate* priv)
{
	if(event->window == priv->bottom_area->window){
		if( (event->y > ((priv->bottom_area->allocation.height-1)-WINDOW_RESIZE_AREA)) &&
			(event->x > ((priv->bottom_area->allocation.width-1)-WINDOW_RESIZE_AREA))){

			if(priv->curser_corner == NULL){
				priv->curser_corner = gdk_cursor_new_for_display(gdk_display_get_default (),GDK_BOTTOM_RIGHT_CORNER);
			}
			gdk_window_set_cursor(priv->window->window, priv->curser_corner);
			return TRUE;
		}
	}
	
	gdk_window_set_cursor(priv->window->window, NULL);

	return FALSE;
}

