/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#ifndef _IO_IPC_H
#define _IO_IPC_H

#include <glib.h>
#include <pthread.h> /** pthread_mutex_* */

#define HAMACHI_SOCK "/.hamachi/ipc_sock"

typedef struct IPCClient{
	gchar *nick;
	gchar *addr;
	gchar *addr_tunnel;
	guint16 port_tunnel;
	gboolean online;	
	struct IPCClient *next;
	struct IPCNetwork *net;
}IPCClient;

typedef struct IPCNetwork{
	gchar *name;	
	gboolean online;
	struct IPCNetwork *next;
	struct IPCClient *clients;
}IPCNetwork;

typedef struct IPCConnection{
	gint socket;
	gint error;
	pthread_mutex_t lock;
}IPCConnection;

typedef struct IPCStatus{
	gchar* useragent;	
	gchar* nick;	
	gint pid;	
	gboolean online;
}IPCStatus;

typedef enum IPC_ERROR{
	IPC_ERROR_NO_ERROR,
	IPC_ERROR_SOCKET,
	IPC_ERROR_ARG,
	IPC_ERROR_PARSE,
	IPC_ERROR_CONNECT,	
	IPC_ERROR_OTHER,	
	IPC_ERROR_WRONG_PASSWD,
	IPC_ERROR_NON_EXISTENT_NETWORK,
	IPC_ERROR_NOT_OWNER,
	IPC_ERROR_ALREADY_ONLINE,
	IPC_ERROR_ALREADY_OFFLINE,
	IPC_ERROR_NETWORK_EXIST
}IPC_ERROR;

typedef enum IPC_OP_CODES{
	IPC_SHUTDOWN 	= 1,
	IPC_LOGIN		= 2,
	IPC_LOGOUT		= 3,
	IPC_CREATE		= 4,
	IPC_DELETE		= 5,
	IPC_JOIN		= 6,
	IPC_STATUS	 	= 7,
	IPC_GET_LIST 	= 8,
	IPC_LEAVE		= 9,
	IPC_GET_NICK	= 10,
	IPC_EVICT 		= 11
	//IPC_SET_NICK
}IPC_OP_CODES;

gchar* 			ipc_get_sock_path	();
IPCConnection*	ipc_connect			(gchar* path);
void			ipc_disconnect		(IPCConnection* connection);
void			ipc_shutdown		(IPCConnection* connection);
const gchar*	ipc_error_string	(IPCConnection *connection);

IPCNetwork*		ipc_list_get		(IPCConnection* connection);
void 			ipc_list_nicks_get	(IPCConnection* connection);
gboolean		ipc_list_nicks_check(IPCNetwork* net);
void			ipc_list_print		(IPCNetwork* net);
void			ipc_list_free		(IPCNetwork* net);
gboolean		ipc_list_equal		(IPCNetwork* net1, IPCNetwork* net2);

IPCStatus*		ipc_status_get		(IPCConnection* connection);
void			ipc_status_print	(IPCStatus *status);
void			ipc_status_free		(IPCStatus *status);

void 			ipc_network_evict	(IPCConnection* connection, gchar* net, gchar* ip);
void			ipc_network_status	(IPCConnection* connection, gchar* net, gboolean online);
void			ipc_network_join	(IPCConnection* connection, gchar* name, gchar* passwd);
void			ipc_network_create	(IPCConnection* connection, gchar* name, gchar* passwd);
void 			ipc_network_delete	(IPCConnection* connection, gchar* name);
void 			ipc_network_leave	(IPCConnection* connection, gchar* name);

#endif
