/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>
#include <pthread.h>

#include "gui_top_area.h"
#include "gui_bottom_area.h"
#include "gui_middle_area.h"
#include "gui_status_icon.h"
#include "gui_frame.h"
#include "gui_popup_menu.h"
#include "io_update.h"
#include "io_ipc.h"
#include "callbacks.h"
#include "main.h"

static MainPrivate* priv;

static void
main_signals_setup()
{      
       g_signal_connect (priv->window, "destroy",
                  G_CALLBACK (gtk_main_quit), NULL);

       g_signal_connect_after (G_OBJECT (priv->window), "button_press_event",
                 G_CALLBACK (main_window_button_press), priv);

       g_signal_connect_after (G_OBJECT (priv->window), "motion_notify_event",
                 G_CALLBACK (main_window_motion_notify), priv);

       g_signal_connect (priv->top_area, "left-button-clicked",
                  G_CALLBACK (top_area_left_button_clicked), priv);

       g_signal_connect (priv->top_area, "right-button-clicked",
                  G_CALLBACK (top_area_right_button_clicked), priv);

       g_signal_connect (priv->bottom_area, "stop_button_clicked",
                  G_CALLBACK (bottom_area_stop_button_clicked),  priv);

       g_signal_connect (priv->bottom_area, "middle_button_clicked",
                  G_CALLBACK (bottom_area_middle_button_clicked),  priv);

       g_signal_connect (priv->middle_area, "network-row-activated",
                  G_CALLBACK (network_row_activated), priv);

       g_signal_connect (priv->middle_area, "client-row-activated",
                  G_CALLBACK (client_row_activated), priv);
}

static void
main_setup()
{
       priv->connection   = NULL;
       priv->window            = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       priv->top_area        = gui_top_area_new ();
       priv->bottom_area  = gui_bottom_area_new ();
       priv->middle_area  = gui_middle_area_new ();

#if (GTK_MINOR_VERSION >= 10)
       priv->status = gui_status_icon_create(priv, TRUE);
#endif

       gui_bottom_area_set_hamachi_status(GUI_BOTTOM_AREA(priv->bottom_area), TRUE);
       
       gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), "Initiating...");

       gtk_window_set_resizable      (GTK_WINDOW(priv->window), TRUE);
       gtk_window_set_decorated      (GTK_WINDOW(priv->window), FALSE);
       gtk_window_set_title   (GTK_WINDOW(priv->window), "Hamachi");
}

int
main (int argc, char **argv)
{
       GtkWidget *vbox;
       GThread *thread;

       priv = g_malloc0(sizeof(MainPrivate));

       g_thread_init(NULL);
    gdk_threads_init();
       gtk_init (&argc, &argv);

       main_setup();

       vbox = gtk_vbox_new(FALSE, 0);

       gtk_box_pack_start(GTK_BOX (vbox), priv->top_area, FALSE, FALSE, 0);       
       
       gtk_container_add (GTK_CONTAINER (vbox), priv->middle_area);
       gtk_box_pack_end  (GTK_BOX (vbox), priv->bottom_area, FALSE, FALSE, 0);
       gtk_container_add (GTK_CONTAINER (priv->window), vbox);

       main_signals_setup();

       gtk_widget_set_size_request (priv->top_area, WIDTH, TOP_HIGHT);
       gtk_widget_set_size_request (priv->bottom_area, WIDTH, BOTTOM_HIGHT);
       thread = g_thread_create ((void*)io_update_loop, (void*)priv, FALSE, NULL);
       gtk_widget_show_all (priv->window);

       gdk_threads_enter();
    gtk_main();
    gdk_threads_leave();

       return 0;
}
